% K-means clustering.
%
% Inputs:
%   - x: a DxN matrix of N data points in a D-dimensional space
%   - m: a DxK matrix with K initial values for the mixture means or, if
%       a scalar, the number K of desired clusters
%   - tol: iteration stops when the overall rms change in coordinates
%       falls below tol, or when maxiter iterations are exceeded,
%       whichever comes first
%   - maxiter: maximum number of iterations
%
% Note: all parameters other than x and m are optional. Default values:
%   - m : initialized by initClusters(x)
%   - tol: 1.0e-6 times the standard deviation of the input point coordinates
%   - maxiter: 100
%
% Outputs:
%   - m: a DxK matrix of K cluster centers. The output K may be smaller
%       than the input K
%   - class: a vector of N membership indices, such that x(:, class(k)) is
%       closest to cluster center m(k)
%   - niter: number of iterations to convergence. If negative, no convergence occurred
%       within maxiter iterations. In that case, niter is set to -maxiter

function [m, class, niter] = kmeans(x, m, tol, maxiter)

if nargin < 2
    error('Must provide at least a matrix of data points and a desired number of clusters')
end

if nargin < 3
    tol = [];
end

if nargin < 4
    maxiter = [];
end

if isempty(tol)
    tol = std(x(:)) * 1.0e-6;
end

if isempty(maxiter)
    maxiter = 100;
end

% Number of clusters
if max(size(m)) == 1
    % m is really K
    K = m;
    m = initClusters(x, K);
else
    K = size(m, 2);
end

checkSizes(x, K, m);

niter = 1;
while 1    
    % Compute class memberships
    class = nearest(m, x);
    
   % Remember old means for convergence check
    oldm = m;

    % Compute means
    m = centroids(x, class);
    
    % Check for convergence
    if size(m, 2) == size(oldm, 2)  % No centroids have disappeared
        if norm(oldm(:) - m(:)) < tol     % Centroids have not moved much
            break;
        end
    end
    
    % Too many iterations?
    niter = niter + 1;
    if niter >= maxiter
        disp(sprintf('Warning: no convergence within %d iterations', maxiter))
        niter = -maxiter;
        break;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function c = nearest(m, x)
c = dsearchn(m', x');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function m = centroids(x, c)

n = max(c);
q = 1;
for k = 1:n
    ck = find(c == k);
    if ~isempty(ck)
        xk = x(:, ck);
        m(:, q) = colmean(xk')';
        q = q + 1;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mc = missing(c, K)

mc = [];
for k = 1:K
    if length(find(c == k)) == 0
        mc = [mc k];
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f = far(x, class, m)

diff = x - m(:, class);
d = colsum(diff .^ 2);
[dmax, n] = max(d);
f = x(:, n);