% Draws N points from a mixture of Gaussians
%
% Inputs:
%   - N: desired number of points
%   - p: a vector of K numbers proportional to the fraction of points
%       desired from each mixture component. Or, if a scalar, the number of
%       components desired. In that case, components are equiprobable
%   - m: a DxK matrix of mixture means. Or, if a scalar, the dimension D of
%       the space
%   - sigma: a vector of K standard deviations for the mixture components
%
% Note: sigma is optional. If not provided, it is initialized at
% random. The same holds for m, if the corresponding argument is a scalar
% (so that it represents the dimension D of the space)
%
% Outputs:
%   - x: a DxN matrix of points
%   - p: a vector with the K actual fractions of points in each component
%   - m: a DxK matrix with the sample means of the K mixture components
%   - sigma: a vector with the sample standard deviations of the K
%       mixture components
%   - class: vector of N membership indices, such that x(:, class(k)) was
%       drawn from mixture component number k

function [x, p, m, sigma, class] = makeClusters(N, p, m, sigma)

if nargin < 3
    error('At least the number of points, number of mixture components, and space dimension must be provided')
end

if max(size(p)) == 1
    % p is really K
    K = p;
    p = 1/K * ones(1, K);
else
    K = length(p);
    p = p(:)' / sum(p);
end

if max(size(m)) == 1
    % m is really D
    D = m;
    m = rand(D, K);
else
    D = size(m, 1);
end

if nargin < 4
	% Spread of each cluster
	spread = 0.03;
	minspread = 0.08;
	sigma = minspread + rand(1, K) * spread;
end

checkSizes(N, p, m, sigma);

% Class memberships in the desired proportions
r = rand(1, N);
c = cumsum(p);
c = [0 c];
class = zeros(1, N);
for k = 1:K
    list{k} = find(r >= c(k) & r < c(k + 1));
    class(list{k}) = k;
end

% Generate the points and sample statistics
x = randn(D, N);
for k = 1:K
    index = list{k};
    y = m(:, k) * ones(1, length(index)) + sigma(k) * x(:, index);
    x(:, list{k}) = y;
    m(:, k) = colmean(y')';
    sigma(k) = sqrt(sum(colstd(y') .^ 2) / D);
    p(k) = length(index) / N;
end